# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['notions', 'notions.cli', 'notions.models']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1,<6.0.0',
 'coloredlogs>=15.0.1,<16.0.0',
 'furl>=2.1.2,<3.0.0',
 'httpx>=0.18.2,<0.19.0',
 'pydantic>=1.8.2,<2.0.0',
 'typer>=0.3.2,<0.4.0',
 'types-PyYAML>=5.4.3,<6.0.0']

entry_points = \
{'console_scripts': ['notions = notions.cli.main:app']}

setup_kwargs = {
    'name': 'notions',
    'version': '0.1.1a0',
    'description': 'Client library for Notion API',
    'long_description': '# Notions - A Python Client Library and CLI for Notion\n\nA command line client and API library for [Notion](https://notion.so).\n\nUses the [Notion API](https://developers.notion.com) to communicate with your Notion pages and databases.\n\n- [Notions - A Python Client Library and CLI for Notion](#notions---a-python-client-library-and-cli-for-notion)\n- [Installation](#installation)\n  - [pipx for CLI usage anywhere](#pipx-for-cli-usage-anywhere)\n  - [pip](#pip)\n  - [poetry](#poetry)\n  - [pipenv](#pipenv)\n- [Authentication](#authentication)\n- [Command Line Usage](#command-line-usage)\n  - [Output Formats](#output-formats)\n  - [Commands](#commands)\n    - [notions api](#notions-api)\n    - [notions search](#notions-search)\n    - [notions database list](#notions-database-list)\n    - [notions database query](#notions-database-query)\n    - [notions page get](#notions-page-get)\n- [API](#api)\n- [Developing Notions](#developing-notions)\n\n# Installation\n\n## pipx for CLI usage anywhere\n\nTo install as a CLI and make it available on your PATH you can use [pipx](https://pypa.github.io/pipx/) to install:\n\n```sh\npipx install notions\n```\n\n## pip\n\nTo install using [pip](https://pip.pypa.io/en/stable/):\n\n```sh\npip install notions\n```\n\n## poetry\n\nTo add as a depdency using [poetry](https://python-poetry.org):\n\n```sh\npoetry add notions\n```\n\n## pipenv\n\nTo add as a dependency using [pipenv](https://pipenv.pypa.io/en/latest/):\n\n```sh\npipenv install notions\n```\n\n# Authentication\n\nCurrently only authentication using an Notion API token is supported (see [Notion\'s Getting Started Guide](https://developers.notion.com/docs#getting-started) for full details).\n\nRough steps to get a token you can use:\n1. Go to https://www.notion.so/my-integrations\n2. Create a new internal integration\n3. Note down the API key, you\'ll need that later\n4. Go to a Database or page in your notion (a top level item in your sidebar) and click on the sharing button\n5. Add your integration to the shares. You need to do this for every database you want to share, Notion doesn\'t expose everything by default.\n\n# Command Line Usage\n\n1. Authenticate first, see [Authentication](#authentication), and get your API token (we\'ll refer to it as `YOUR_KEY_GOES_HERE` below)\n2. You can either pass in the key each time you call the client using `notions --notion-api-key YOUR_KEY_GOES_HERE` or set the `NOTION_API_KEY` environment variable.\n   1. To set in fish: `set -Ux NOTION_API_KEY YOUR_KEY_GOES_HERE`\n   2. To set in bash/zsh: `export NOTION_API_KEY=YOUR_KEY_GOES_HERE`\n3. Call `notions --help` to explore your options\n\nSome options are global and go before the command, e.g. `notions --output-format notion_yaml database query --query foo`.\n\n## Output Formats\n\nFormats you can specify with `notions --output-format`\n1. `notion_json` - Parses API output and dumps back as JSON, with multiple items contained in a list.\n2. `notion_jsonl` - Parses API output and dumps back as JSON, one JSON object per line.\n3. `notion_yaml` - Parses API output and dumps back as YAML, with multiple items in a list.\n4. `text` (default) - Parse API output and return a simple text representation. Handy for looking up page and database ids.\n\n## Commands\n\n### notions api\n\n`notions api` allows you to issue requests using the authentication features of the client.\n\n1. To make a request and get back the response JSON: `notions api METHOD PATH`.\n2. To make a request and leverage the pagination support (this will parse the response and give you the option to render as YAML or other formats): `notions api --paginate METHOD PATH`. Note that due to how the flags are handled you need the `--paginate` to trigger parsing and allow for different output formats.\n\n```sh\n# Get all databases, without pagination and return raw JSON from Notion\'s API\nnotions api GET /v1/databases/\n\n# Get all databases with pagination and format output as YAML\n# This is the same as `notions --output-format notion_yaml database list`\nnotions --output-format notion_yaml api GET /v1/databases/ --paginate\n```\n\n### notions search\n\n`notions search` allows you to search across all databases and pages you have shared with the Notion integration you created. You can restrict the search with `--query SOME_QUERY_STRING` and items which have a matching title will be returned.\n\n```sh\n# show all pages and dbs with "example" in their title\nnotions search --query example\n```\n\n### notions database list\n\n`notions database list` lists all the databases the integration has access to.\n\n### notions database query\n\nRun a query against all pages in a given database. You\'ll need the database UUID from something like `notions database list`.\n\n```sh\n# Show all pages in the given db and sort by the HP property in descending order\nnotions database query cc5ef123-05f5-409e-9b34-38043df965b0 --sort-property HP:descending\n```\n\n### notions page get\n\nGet a single page.\n\n```sh\n# Get a page in YAML\nnotions --output-format notion_yaml page get ccad10e7-c776-423e-9662-6ad5fb1256d6\n```\n\n# API\n\nThe main way to use the library is via the HTTP client (see [examples/basic.py](examples/basic.py) for same code):\n\n\n```python\n"""Example showing basic use of the client for API calls\n"""\n\nimport asyncio\nimport os\nimport typing\n\nimport httpx\n\nfrom notions.client import NotionAsyncClient\nfrom notions.models.database import Database\n\n\nasync def http_list_databases(notion_client: NotionAsyncClient) -> httpx.Response:\n    """Do a HTTP call with authentication to list databases\n\n    This uses notions as a convenience for HTTP calls using httpx\n    """\n    async with notion_client.async_client() as client:\n        url = notion_client.get_url_for_path("/v1/databases")\n        return await client.request("GET", url.url)\n\n\nasync def list_databases(\n    notion_client: NotionAsyncClient,\n) -> typing.AsyncIterable[Database]:\n    """Do a list db call (automatically reads all pages)\n\n    This is the same underlying call as `http_list_databases` above\n    but will also parse the response and handle pagination.\n\n    Will yield notions.models.database.Database\n    """\n    async for db in notion_client.list_databases():\n        yield db\n\n\nasync def main():\n    api_token = os.environ.get("NOTION_API_KEY", "no-token-set")\n    notion_client = NotionAsyncClient(api_token)\n\n    response = await http_list_databases(notion_client)\n    print(response)\n    print(response.json())\n    response.raise_for_status()\n\n    async for db in list_databases(notion_client):\n        print(db.id)\n        print(db.title)\n        print(db)\n\n\nif __name__ == "__main__":\n    asyncio.run(main())\n\n```\n\n# Developing Notions\n\nTo develop locally you\'ll need python 3.6+, [poetry](https://python-poetry.org) and make:\n\n1. `make dep` to set up the development environment.\n2. `make test` to test all supported pythons or `make pytest` to run tests more directly for your python.\n\nYou\'ll need a plan which allows for creating Notion integrations (API access) to develop against Notion, but I\'m going to assume you have that if you\'re interested in this library :D\n\nTo release ensure you\'re happy with the current state of main and run `make release`.\n',
    'author': 'Michael Twomey',
    'author_email': 'mick@twomeylee.name',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/micktwomey/notions',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
